/**
 * Create an drag and drop uploader.
 *
 * The `.presentationUiDnduploader()` method creates a drag and drop uploader.
 *
 * Only applicable to <input type="file"> nodes.
 *
 * ### Examples
 * Explanation for example,
 *
 * JavaScript:
 *
 *
 *     <input type="file" name="NewFile" class="ep-js" data-js="de_epages.presentationUiDnduploader({
 *       multiple:false,
 *       accept:'image/*',
 *       data:{
 *         ChangeAction:'SaveImage',
 *         ObjectID:#ObjectID,
 *         SCALE_WIDTH_l:#Shop.ProductImageLargeWidth,
 *         SCALE_HEIGHT_l:#Shop.ProductImageLargeHeight,
 *         SCALE_WIDTH_m:200,
 *         SCALE_WIDTH_s:100,
 *         SCALE_HEIGHT_s:100,
 *         SCALE_WIDTH_h:150,
 *         SCALE_HEIGHT_h:150,
 *         SCALE_WIDTH_xs:50,
 *         SCALE_HEIGHT_xs:33,
 *         SCALE_NAME_m:'ImageMedium',
 *         SCALE_NAME_s:'ImageSmall',
 *         SCALE_NAME_h:'ImageHotDeal'
 *       },
 *       callback:'?ViewAction='+ep.config.viewAction+'&ObjectID='+ep.config.objectId+'&DialogArea=ViewArea'
 *     })"/>
 *
 *
 * @class jQuery.ui.presentationUiDnduploader
 * @extends jQuery.widget
 *
 * @uses jQuery.dict
 * @uses jQuery.tmpl
 * @uses jQuery.i18n
 * @uses jQuery.expr
 * @uses jQuery.fn.form
 * @uses jQuery.ui.widget
 * @uses jQuery.ui.progressbar
 * @uses de_epages
 * @uses ep.ajaxTransport
 * @uses ep.ui.dialog
 * @since 6.17.38
 */

/**
 * @cfg {String} [accept] A string of MIME types and/or MIME type groups.
 */

/**
 * @cfg {String} [url] A string containing the URL to which the request is sent.
 */

/**
 * @cfg {String} [buttonText] A string for the upload button
 */

/**
 * @cfg {Object} [data] A map of additional data to send with the request.
 */

/**
 * @cfg {Function,String} [callback] A url to open or method to call when upload process is finished. The callback method receives data and statusText as arguments.
 */

/**
 * @cfg {Boolean} [multiple] A boolean indication whether to allow muplitple files to upload.
 */

/**
 * @cfg {Boolean} [big] A boolean boolean short handle set style big.
 */

/**
 * @cfg {String} [style] One of the available styles: normal,big,icon.
 */

/**
 * @cfg {Array} [exists] An array including already existing files.
 */

/**
 * See `jQuery.ui.presentationUiDnduploader` for details.
 *
 * @param {Object} [options] A map of additional options pass to the method.
 * @param {String} [accept] A string of MIME types and/or MIME type groups.
 * @param {String} [url] A string containing the URL to which the request is sent.
 * @param {String} [buttonText] A string for the upload button
 * @param {Object} [data] A map of additional data to send with the request.
 * @param {Function,String} [callback] A url to open or method to call when upload process is finished. The callback method receives data and statusText as arguments.
 * @param {Boolean} [multiple] A boolean indication whether to allow muplitple files to upload.
 * @param {Boolean} [big] A boolean boolean short handle set style big.
 * @param {String} [style] One of the available styles: normal,big,icon.
 * @param {Array} [exists] An array including already existing files.
 *
 * @method presentationUiDnduploader
 * @member jQuery
 *
 * @since 6.12.0
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.presentation.ui.dnduploader
 */

define( "de_epages/presentation/ui/dnduploader", [
    "jquery",
    "ep",
    "de_epages",
    "util/mime",
    "$dict!../dictionary",
    "$tmpl!./dnduploader",
    "$tmpl!./dnduploader-inputs",
    "$tmpl!./dnduploader-sortable_item",
    "$tmpl!./dnduploader-mainplaceholder",
    "$tmpl!./dnduploader-alert",

    "jquery/ui/widget",
    "jquery/tmpl",
    "jquery/ui/sortable",
    "ep/fn/busy",
    "ep/alert"
], function ($, ep, de_epages, mime, presentationDict, tmplDndUploader, tmplDndUploaderInputs, tmplDndUploaderSortableItem, tmplDndUploaderMainPlaceholder, tmplDndUploaderAlert) {
    "use strict";

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Error}
     *                  {AddImages}
     *                  {Notification}
     *
     * @dictionary      de_epages.presentation.dictionary
     *
     * @translation     {FilesDoNotExist}
     *                  {FileWriteError}
     *                  {UploaderError}
     *                  {FileExistsError}
     *                  {FileDeleteError}
     *                  {FileSizeLimitError}
     *                  {MimetypeNotAcceptError}
     *                  {UnknownError}
     *                  {InvalidImageFile}
     *                  {FileSizeLimitError}
     */

    // translation object
    var translationsDnduploader = {
        FILE_WRITE_ERROR: presentationDict.translate('FileWriteError'),
        FILE_SCALE_ERROR: presentationDict.translate('FileWriteError'),
        FILE_EXISTS_ERROR: presentationDict.translate('FileExistsError'),
        FILE_DELETE_ERROR: presentationDict.translate('FileDeleteError'),

        Error: presentationDict.translate('Error'),
        filesDoNotExist: presentationDict.translate('FilesDoNotExist'),
        uploadError: presentationDict.translate('UploaderError'),
        uploadErrorMsg: presentationDict.translate('FileUploadError'),
        invalidImageFile: presentationDict.translate('InvalidImageFile'),
        mimetypeNotAcceptError: presentationDict.translate('MimetypeNotAcceptError'),
        fileSizeLimitError: presentationDict.translate('FileSizeLimitError'),

        addImages: presentationDict.translate('AddImages'),
        notification: presentationDict.translate('Notification')
    };

    $.widget('ui.presentationUiDnduploader', {
    //  dropZone: {},

        files: {
            order: [],
            exist: [],
            upload: [],
            mime_error: [],
            size_error: []
        },

        selectors: {
            sortableItem: 'li.ep-dropzone-sortable-item'
        },

        // fileCount,

        options: {
        //    files :{exist: [], order: []},
            newProduct: true,
            mime: 'image/(gif|png|jpeg|bmp)',
            multiple: true,
            objectID: 1,
            maxFileSize: 4096,
            dimensions: {
                width: 150,
                height: 100
            }
        },

        _create: function () {
            var self = this,
                elem = self.element,
                o = self.options,
                renderedTmpl;

            // render base template for dnd-uploader
            renderedTmpl = tmplDndUploader([{
                multiple: o.multiple,
                order: JSON.stringify((o.files && o.files.order) ? o.files.order : []),
                tAddImages: translationsDnduploader.addImages
            }]);

            // initialize object for dropZone elements
            self.dropZone = {};

            // load template and push named tmpl elements to current instance
            $.extend(self.dropZone, renderedTmpl.tmplItem('elements'));
            // append base template
            elem.append(renderedTmpl);

            // register handler for drag and drop and file input
            // do not allow dropping on body
            $('body').on('dragover', function (e) {
                e.stopPropagation();
                e.preventDefault();

                e.originalEvent.dataTransfer.dropEffect = 'none';
            });
            // DragOver-Handler
            self.dropZone.area.on('dragover', $.proxy(function (e) {
                self._handleDragOver(e);
            }, self));
            // Drop-Handler
            self.dropZone.area.on('drop', $.proxy(function (e) {
                self._handleDrop(e);
            }, self));
            // fileInput-Handler
            self.dropZone.fileInput.on('change', $.proxy(function (e) {
                self._handleFile(e);
            }, self));

            // initialize sortable list
            self._initSortable();

            // if a files object was handed over to the widget
            if (o.files) {
                // apply widget files object
                $.extend(self.files, o.files);

                // inserts already existing files
                if (self.files.exist.length) {
                    // display busy layer while loading images
                    elem.busy('show');

                    // insert regular images
                    // and hand over a callback that will be executed after all regular images are loaded
                    self._insertExistingImages(self._handleUploadDataUrlImages);
                }

                // inserts files handed over as DataURL (only if no regular images exist)
                if (self.files.exist.length===0 && self.files.upload.length) {
                    self._handleUploadDataUrlImages();
                }
            }

            self._setMainImage();
        },

        _insertExistingImages: function (callback) {
            var self = this,
                o = self.options,
                $img,
                list = [],
                i;

            // reset file count (needed to indicate if all files are loaded)
            self.fileCount = self.files.exist.length;

            // loop over all existing files
            for (i=0; i<self.files.exist.length; i++) {

                $img = $('<img />');

                // write dataUrl to image source
                $img
                    .attr('src', self.files.exist[i].m_src)
                    .data('id', self.files.exist[i].id);

                // load handler for image
                $img.on('load', function(evt) {
                    var $innerImg = $(this),
                        id = $innerImg.data('id');

                    // insert item with image to sortable list
                    self._insertSortableItem($innerImg, id);

                    // decrease file counter
                    self.fileCount--;

                    // check uploaded files
                    self._afterExistingImagesLoaded(callback, list);

                });


                // error handler for image
                $img.on('error', function(evt) {
                    var $innerImg = $(this),
                        id = $innerImg.data('id');

                    // decrease file counter
                    self.fileCount--;

                    // push faulty file to error message list
                    list.push($innerImg.attr('src') + " (id: " + id + ")");

                    // check uploaded files
                    self._afterExistingImagesLoaded(callback, list);

                });

            }
        },

        _afterExistingImagesLoaded: function (callback, list) {
            var self = this,
                elem = self.element;

            // an error occurs after the last uploaded element
            if (self.fileCount === 0) {
                // if an error occurs during loading
                if (list.length > 0) {
                    // open message dialog
                    self._messageDialog(translationsDnduploader.filesDoNotExist + ": ", list);
                }

                // sort items by order array
                self._sortListByArray();
                self.dropZone.sortable.sortable('refresh');
                self._setMainImage();

                // remove busy layer
                elem.busy('hide');

                // execute callback after all regular images are loaded
                if (callback) {
                    callback.call(self);
                }
            }

        },

        _handleUploadDataUrlImages: function () {
            var self = this,
                i;

            // reset file count (needed to indicate if all files are loaded)
            self.fileCount = self.files.upload.length;

            if (self.files.upload_order && self.files.upload_order.length) {
                // concatinate order and upload order arrays
                self.files.order = self.files.order.concat(self.files.upload_order);
            }

            // loop over upload images
            for (i=0; i<self.files.upload.length; i++) {
                // insert image
                self._insertDataUrlImage(self.files.upload[i]);
            }
        },

        _initSortable: function () {
            var self = this,
                o = self.options;

            // initialize sortable list
            self.dropZone.sortable.sortable({
                // options
                helper: "clone",
                placeholder: "ep-dropzone-state-highlight AlignCenter col-xs-12 col-md-6 col-lg-4",
                items: "li:not(.ep-dropzone-placeholder)",
                cancel: "li:first-child",
                opacity: 0.6,
                tolerance: 'pointer',

                // events
                start: function(event, ui) {
                    var list = ui.item.parent().children('li'),
                        len = list.length,
                        i;

                    // loop over list items to identify the placeholder item
                    for (i=0; i<len; i++) {
                        // if placeholder is found => insert position number and leave loop
                        if (list[i] === ui.placeholder[0]) {
                            ui.placeholder.html('<span class="ep-dropzone-position">' + i + '</span>');
                            break;
                        }
                    }
                },

                // events
                update: function(event, ui) {
                    var files = [],
                        ajaxData,
                        order = [];

                    if (!o.newProduct) {
                        // get deep copy of array
                        files = $.map(self.files.exist, function (obj) {
                            return $.extend(true, {}, obj);
                        });

                        // loop over array and remove dataUrl
                        files.forEach(function(item) {
                            delete item.dataUrl;
                        });

                        // build current order array
                        self.dropZone.sortable.find(self.selectors.sortableItem).each(function(key, item){
                            order.push($(item).data('id'));
                        });

                        // build data object for AJAX request
                        ajaxData = $.extend({}, {imgData: files, order: order});
                        // AJAX-Request to delete item
                        ep.ajax({
                            url: '?',
                            type: 'POST',
                            dataType: 'json',
                            data: {
                                'ChangeAction': 'JSONHandleProductImagesUnity',
                                'ObjectID': o.objectID,
                                'Operation': 'order',
                                'data': JSON.stringify(ajaxData)
                            }
                        })
                        .done(function( data, textStatus, jqXHR ) {
                            // rebuild order array
                            self._sortOrder(self.dropZone.sortable.find(self.selectors.sortableItem));

                        })
                        .fail(function( jqXHR, textStatus, errorThrown ) {
                            // console.log('fail (updated)', arguments);
                        });
                    } else {
                        // rebuild order array
                        self._sortOrder(self.dropZone.sortable.find(self.selectors.sortableItem));
                    }
                },

                // events
                change: function(event, ui) {
                    var list = ui.item.parent().children('li'),
                        len = list.length,
                        isGreater = false,
                        i;

                    // loop over list items to identify the placeholder item
                    for (i=0; i<len; i++) {
                        // if placeholder was inserted after the current dragged item
                        if (list[i] === ui.item[0]) {
                            isGreater = true;
                        }

                        // if placeholder is found => insert position number and leave loop
                        if (list[i] === ui.placeholder[0]) {
                            ui.placeholder.html('<span class="ep-dropzone-position">' + (isGreater ? i : i+1) + '</span>');
                            break;
                        }
                    }
                }
            });

            // make sortable list unselectable
            self.dropZone.sortable.disableSelection();
        },

        _insertSortableItem: function ($img, id) {
            var self = this,
                $body = $('body'),
                elements = {},
                dimensions = {},
                itemTmpl;

            // render template for sortable item
            itemTmpl = tmplDndUploaderSortableItem([{
                id: id
            }]);
            // load template and push named tmpl elements to object
            $.extend(elements, itemTmpl.tmplItem('elements'));

            // register click handler for delete button
            elements.deleteButton.on('click', $.proxy(function (evt) {
                self._deleteItemById($(evt.target).data('id'));
            }, self));

            // insert image and template
            elements.itemHolder.append($img);
            self.dropZone.placeholder.before(itemTmpl);
        },

        // rearrange order array depending on the sort order of the sortable list
        _sortOrder: function (list) {
            var self = this,
                o = self.options,
                uploadOrder = [],
                id;

            // reset order array
            self.files.order.length = 0;
            // rebuild order array
            list.each(function(key, item){
                id = $(item).data('id');
                self.files.order.push(id);

                // add id of upload image to array
                if(ep('#ep-dropzone-inputs-' + id).length) {
                    uploadOrder.push(id);
                }
            });

            // fill inputs only for new products (AJAX request otherwise)
            if (o.newProduct) {
                // insert order array as string
                self.dropZone.orderInput.val(JSON.stringify(self.files.order));

                // fill inputs for error handling
                self.dropZone.uploadedDataUrlImages.val(JSON.stringify(self.files.upload));
                self.dropZone.uploadedDataUrlImagesOrder.val(JSON.stringify(uploadOrder));
            }

            self._setMainImage();
        },

        // sorts sortable list items by order of corresponding order array
        // necessary e.g. if depending to the loading duration the order of the sortable list
        // is not equal the order of the files.order array
        _sortListByArray: function () {
            var self = this,
                i;

            if (self.files && self.files.order && self.files.order.length) {
                for (i=0; i<self.files.order.length; i++) {
                    self.dropZone.placeholder.before($('#ep-dropzone-sortable-item-' + self.files.order[i]));
                }
            }
        },

        // set the main image to the first image of the sortable list
        _setMainImage: function () {
            var self = this,
                list = self.dropZone.sortable.find(self.selectors.sortableItem),
                renderedMain,
                elements = {};

            self.dropZone.mainImage.empty();

            if (list.length > 0) {
                self.dropZone.mainImage.addClass('with-content');
                self.dropZone.mainImage.append($('<img src=' + list.eq(0).find('img').attr('src') + ' />'));
            } else {
                renderedMain = tmplDndUploaderMainPlaceholder();
                // load template and push named tmpl elements to current instance
                $.extend(elements, renderedMain.tmplItem('elements'));
                self.dropZone.mainImage.append(elements.mainImagePlaceholder);
                self.dropZone.mainImage.removeClass('with-content');
            }
        },

        // DnD
        _handleDragOver: function(evt) {
            evt.stopPropagation();
            evt.preventDefault();

            // set effect to copy
            evt.originalEvent.dataTransfer.dropEffect = 'copy';
        },

        // Drop-Handler
        _handleDrop: function(evt) {
            evt.stopPropagation();
            evt.preventDefault();

            var self = this;

            self._manageFiles(evt.originalEvent.dataTransfer.files);
        },

        _handleFile: function(evt) {
            var self = this;

            self._manageFiles(evt.originalTarget.files);
        },

        _manageFiles: function (files) {
            var self = this,
                o = self.options,
                len = o.multiple ? files.length : 1,
                i;

            // clear error array
            self.files.mime_error.length = 0;
            self.files.size_error.length = 0;
            self.fileCount = len;
            self.imageLength = 0;

            // loop over all files and send them to a fileReader
            for (i=0; i<len; i++) {
                self._fileReader(files[i]);
            }
        },

        // FileReader
        _fileReader: function(file) {
            var self = this,
                reader = new FileReader();

            reader.onload = function (evt) {
                var fileReader = this;

                // call handler to process loaded file
                self._readerSuccessHandler(fileReader, file, evt);
            };

            reader.onloadstart = function (evt) {
                var errObj;

                // if a maximum file size error occurs
                if (Math.round(file.size/1000, 0) > self.options.maxFileSize) {
                    errObj = {
                        array: self.files.size_error,
                        translation: 'fileSizeLimitError'
                    };
                // if a mime type error occurs
                } else if (!file.type.match(self.options.mime)) {
                    errObj = {
                        array: self.files.mime_error,
                        translation: 'mimetypeNotAcceptError'
                    };
                }

                // if an error occurs
                if (errObj !== undefined) {
                    // push file to error object
                    errObj.array.push({
                        id: "id_" + file.size + new Date().getTime(),
                        name: file.name
                    });

                    self.fileCount--;

                    // an error occurs and only one element should be uploaded
                    if (self.fileCount === 0 && self.imageLength === 0) {
                        // if file size errors occured during the readAsDataUrl method
                        if (self.files.size_error.length > 0) {
                            // display error message
                            self._errorMessage({
                                array: self.files.size_error,
                                translation: 'fileSizeLimitError'
                            });
                        }

                        // if mime type errors occured during the readAsDataUrl method
                        if (self.files.mime_error.length > 0) {
                            // display error message
                            self._errorMessage({
                                array: self.files.mime_error,
                                translation: 'mimetypeNotAcceptError'
                            });
                        }

                        // reset file input
                        self.dropZone.fileInput.val("");
                    }

                    reader.abort();

                    return;
                }
            };

            // reader.onprogress = readerProgressHandler;
            // reader.onerror = readerErrorHandler;

            // read file as dataUrl
            reader.readAsDataURL(file);

        },

        // processes a file loaded by a fileReader
        _readerSuccessHandler: function(fileReader, file, evt) {
            var self = this,
                o = self.options,
                id = "id_" + file.size + new Date().getTime(),
                fileData;

            // increase sucessfully loaded file counter
            self.imageLength++;

            // write file informations to object
            fileData = {
                id: id,
                name: file.name.replace(/_/g, "-"),
                dataUrl: fileReader.result
            };

            // write informations for file on object
            self.files.upload.push(fileData);

            // call method to insert file
            self._insertDataUrlImage(fileData);
        },


        // inserts a data url image
        _insertDataUrlImage: function (fileData) {
            var self = this,
                elem = self.element,
                o = self.options,
                $img = $('<img />'),
                ajaxData;

            // write dataUrl to image source
            $img.attr('src', fileData.dataUrl);

            // load handler for image
            $img.on('load', function(evt) {
                var $img = $(this),
                    inputTmpl;

                // counts how many files are left to load
                self.fileCount--;

                // insert item with image to sortable list
                self._insertSortableItem($img, fileData.id);

                //
                if (o.newProduct) {
                    // insert input template for image information
                    inputTmpl = tmplDndUploaderInputs([fileData]);
                    self.dropZone.holder.append(inputTmpl);
                }


                // an error occurs after the last uploaded element
                if (self.fileCount === 0) {

                    if (o.newProduct) {
                        self._afterDropLoad();
                    } else {
                        // rebuild order array
                        self._sortOrder(self.dropZone.sortable.find(self.selectors.sortableItem));

                        // display busy layer
                        elem.busy('show');
                        // build data object for AJAX request
                        ajaxData = $.extend({}, {imgData: self.files.upload, order: self.files.order});
                        // self.files.upload (= new uploaded files)
                        ep.ajax({
                            url: '?',
                            type: 'POST',
                            dataType: 'json',
                            data: {
                                'ChangeAction': 'JSONHandleProductImagesUnity',
                                'ObjectID': o.objectID,
                                'Operation': 'save',
                                'data': JSON.stringify(ajaxData)
                            }
                        })
                        .done(function(data, textStatus, jqXHR) {
                            var _that = self,
                                list = [];

                            // error handling
                            if (data.error === true) {
                                // loop over array and remove dataUrl
                                data.failed.forEach(function(error) {
                                    // remove sortable item
                                    $('#ep-dropzone-sortable-item-' + error.id).remove();

                                    // remove id from order list
                                    self.files.order.splice(self.files.order.indexOf(error.id), 1);

                                    self.files.upload.forEach(function (elem, index) {
                                        if (elem.id === error.id) {
                                            // delete elem;
                                            _that.files.upload.splice(index, 1);
                                        }
                                    });

                                    // add to message list
                                    list.push('<strong class="DisplayBlock">' + (error.name!==undefined ? error.name : error.source) + ':</strong>' + translationsDnduploader[error.reason]);
                                });

                                // open message dialog
                                self._messageDialog(translationsDnduploader.uploadError, list);
                                // refresh sortable list
                                self.dropZone.sortable.sortable('refresh');
                            }

                            // callback
                            self._afterDropLoad();

                            // merge upload array to exists array
                            $.merge(self.files.exist, self.files.upload);
                            // delete newly uploaded files
                            self.files.upload.length = 0;
                        })
                        .fail(function(jqXHR, textStatus, errorThrown) {
                            // error handling
                            // loop over upload array
                            self.files.upload.forEach(function(item) {
                                // remove sortable item
                                $('#ep-dropzone-sortable-item-' + item.id).remove();

                                // remove id from order list
                                self.files.order.splice(self.files.order.indexOf(item.id), 1);
                            });

                            // refresh sortable list
                            self.dropZone.sortable.sortable('refresh');

                            // delete newly uploaded files
                            self.files.upload.length = 0;

                            // open message dialog
                            if (textStatus === "error") {
                                self._messageDialog(translationsDnduploader.uploadError, [errorThrown]);
                            } else {
                                self._messageDialog(translationsDnduploader.uploadError, [translationsDnduploader.uploadErrorMsg]);
                            }
                        })
                        .always(function(data, textStatus, jqXHR) {
                            // hide busy layer
                            elem.busy('hide');
                        });
                    }
                }

            });
        },

        // method called after all files are completely loaded
        _afterDropLoad: function() {
            var self = this,
                o = self.options;

            // if an upload array is handed over initially
            if (self.files.upload_order && self.files.upload_order.length) {
                self._sortListByArray();

                // delete order property for upload array to avoid sorting more than one time
                delete self.files.upload_order;
            }

            self.dropZone.sortable.sortable('refresh');
            // rebuild order array
            self._sortOrder(self.dropZone.sortable.find(self.selectors.sortableItem));
            // reset file input
            self.dropZone.fileInput.val("");

            // set the main image
            self._setMainImage();

            // if file size errors occured during the readAsDataUrl method
            if (self.files.size_error.length > 0) {
                // display error message
                self._errorMessage({
                    array: self.files.size_error,
                    translation: 'fileSizeLimitError'
                });
            }

            // if mime type errors occured during the readAsDataUrl method
            if (self.files.mime_error.length > 0) {
                // display error message
                self._errorMessage({
                    array: self.files.mime_error,
                    translation: 'mimetypeNotAcceptError'
                });
            }
        },

        // build a error message list for the handed over error object
        _errorMessage: function (error) {
            var self = this,
                list = [];

            for(var j=0; j<error.array.length; j++) {
                list.push(error.array[j].name);
            }

            // open message dialog
            self._messageDialog(translationsDnduploader[error.translation], list);
        },

        // shows an error dialog
        _messageDialog: function(info, messages, type) {
            var self = this,
                alertTmpl;

            // render alert template
            alertTmpl = tmplDndUploaderAlert([{
                messages: messages,
                info: info
            }]);

            // if already an alert dialog exists
            if (self.alert) {
                // clean up previous dialog and append new message
                self.alert.find('.Message')
                    .append(alertTmpl);
                // open alert dialog
                self.alert.uiDialog('open');
            } else {
                // show alert
                self.alert = ep.alert({
                    title:  translationsDnduploader.notification,
                    type:   type || 'notification',
                    size:   'l',
                    content: alertTmpl,
                    close:  function( event ){
                        // clean up previous dialog
                        self.alert.find('.Message').empty();
                    },
                    ok:     function( event ){
                        // clean up previous dialog
                        self.alert.find('.Message').empty();
                    }
                });
            }
        },

        _readerProgressHandler: function() {
            // ToDo: handler for progress view
            //console.log("readerProgressHandler: ", arguments);
        },

        _readerErrorHandler: function() {
            // ToDo: error handler
            //console.log("readerErrorHandler: ", arguments);
        },

        // recalculated dimensions (maintaining the proportions)
        _recalcDimensions: function(dimensions) {
            var self = this,
                o = self.options,
                factor;

            dimensions.origWidth = dimensions.width;
            dimensions.origHeight = dimensions.height;

            if (dimensions.width > o.dimensions.width) {
                factor = o.dimensions.width / dimensions.width;

                dimensions.width = o.dimensions.width;
                dimensions.height = dimensions.height * factor;
            }

            if (dimensions.height > o.dimensions.height) {
                factor = o.dimensions.height / dimensions.height;

                dimensions.height = o.dimensions.height;
                dimensions.width = dimensions.width * factor;
            }

            dimensions.width = Math.floor(dimensions.width);
            dimensions.height = Math.floor(dimensions.height);

            return dimensions;
        },

        _getItemById: function (id) {
            var self = this,
                i;

            // loop over already existing images
            for (i=0; i<self.files.exist.length; i++) {
                if (self.files.exist[i].id === id){
                    // return self.files.exist[i];
                    return {array: self.files.exist, index: i};
                }
            }

            // loop over new images
            for (i=0; i<self.files.upload.length; i++) {
                if (self.files.upload[i].id === id){
                    // return self.files.upload[i];
                    return {array: self.files.upload, index: i};
                }
            }

            return false;
        },

        _deleteItemById: function (id) {
            var self = this,
                o = self.options,
                sortableItem = $('#ep-dropzone-sortable-item-' + id),
                item,
                delItem,
                ajaxData,
                files,
                order = [];


            // delete item from files array
            item = self._getItemById(id);

            // display busy layer
            sortableItem.busy('show');

            // if the product is not new => use AJAX request
            if (!o.newProduct) {
                // get deep copy of array
                files = $.map(item.array, function (obj) {
                    return $.extend(true, {}, obj);
                });

                // loop over array and remove dataUrl
                files.forEach(function(item) {
                    delete item.dataUrl;
                });

                // get item for deletion
                delItem = files[item.index];
                // delete delItem.dataUrl;

                // build current order array
                order = $.grep(self.files.order, function (id) {
                    return delItem.id !== id;
                });

                // build data object for AJAX request
                ajaxData = $.extend({}, {imgData: files, imgDelete: [delItem], order: order});
                // AJAX-Request to delete item
                ep.ajax({
                    url: '?',
                    type: 'POST',
                    dataType: 'json',
                    data: {
                        'ChangeAction': 'JSONHandleProductImagesUnity',
                        'ObjectID': o.objectID,
                        'Operation': 'delete',
                        'data': JSON.stringify(ajaxData)
                    }
                })
                .done(function(data, textStatus, jqXHR) {
                    var list = [];

                    // error handling
                    if (data.error === true) {
                        // loop over array and remove dataUrl
                        data.failed.forEach(function(error) {
                            // add to message list
                            list.push('<strong class="DisplayBlock">' + (error.name!==undefined ? error.name : error.source) + ':</strong>' + translationsDnduploader[error.reason]);
                        });

                        // open message dialog
                        self._messageDialog(translationsDnduploader.Error, list);

                        // remove busy layer again
                        sortableItem.busy('hide');
                    } else {
                        // callback
                        self._deleteItem({
                            item: item,
                            id: id,
                            sortable: sortableItem
                        });
                    }
                })
                .fail(function(jqXHR, textStatus, errorThrown) {
                    // open message dialog
                    self._messageDialog(translationsDnduploader.Error, [translationsDnduploader.uploadErrorMsg]);

                    // remove busy layer again
                    sortableItem.busy('hide');
                });
            } else {
                self._deleteItem({
                    item: item,
                    id: id,
                    sortable: sortableItem
                });
            }
        },

        _deleteItem: function (opts) {
            var self = this;

            // remove container for image information input from DOM
            $('#ep-dropzone-inputs-' + opts.id).remove();

            // remove busy layer again
            opts.sortable.busy('hide');
            // remove item from sortable list
            opts.sortable.remove();
            // refresh sortable list
            self.dropZone.sortable.sortable('refresh');
            // rebuild order array
            self._sortOrder(self.dropZone.sortable.find(self.selectors.sortableItem));

            // delete item from array
            opts.item.array.splice(opts.item.index, 1);
        },

        /**
         * This method removes the container on which the Uploader has been appended to.
         *
         * @method destroy
         * @member jQuery.ui.presentationUiUploader
         *
         * @since 6.12.0
         */
        destroy: function () {
            var self = this;

            self.elem.off('.presentationUiDnduploader');

            self._superApply(arguments);
        }

    });

    return de_epages;

});
